package Template::Plugin::Catalyst::View::PDF::Reuse;

use base qw(Template::Plugin::Procedural);
use PDF::Reuse;

=head1 NAME

Template::Plugin::Catalyst::View::PDF::Reuse

=head1 SYNOPSYS

Template Toolkit plugin for PDF::Reuse

=head1 AUTHOR

Jon Allen, L<jj@jonallen.info>

=head1 SEE ALSO

Penny's Arcade Open Source - L<http://www.pennysarcade.co.uk/opensource>

=head1 COPYRIGHT & LICENSE

Copyright 2009 Penny's Arcade Limited, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1;
